#!/bin/sh

HASH=md5sum
DECOMP=cat

usage()
{
    echo "Usage: $0 [OPTION] [IP ADDRESS]
Display a checksum of configuration file from JTAG-Blazer's flash memory.

options:
     -h             show this message
     -c COMMAND     specify the program to be used for checksum calculation
     -u             calculate the checksum without decompressing the file"
    exit $1
}

ARGLIST=`getopt hc:u $@`
if [ $? -eq 0 ]; then
    set -- $ARGLIST
else
    exit 1
fi

while [ $# != 0 ]; do
    case $1 in
	-h)
	    usage 0
	    ;;
	-c)
	    shift
	    HASH=$1
	    ;;
	-u)
	    DECOMPFLG=1
	    ;;
	--)
	    shift
	    IPADDR=$1
	    ;;
    esac
    shift
done

if [ -z "$IPADDR" ]; then
    echo 1>&2 "$0: IP address is not specified"
    usage 1
fi

COMP=`jbfiledata $IPADDR | grep Compression`
if [ -z "$COMP" ]; then
    exit 2
fi

if [ `echo $COMP | cut -d ' ' -f3` = ON ]; then
    if [ -z "$DECOMPFLG" ]; then
	DECOMP=jbdecomp
    fi
fi

TMPFILE=`mktemp -t tmp.XXXXXXXXXX`
jbgetfile $IPADDR $TMPFILE
if [ $? -eq 0 ]; then
    $DECOMP $TMPFILE | $HASH
fi

rm -f $TMPFILE
