#include <iostream>
#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "jblib.h"

void usage()
{
	cout << "Usage: jbprgosize [IP ADDRESS] [OPTION]" << endl;
	cout << "Show writable firmware size in JTAG-Blazer" << endl;
	cout << " -h,\tshow this message" << endl;
}

int main(int argc, char** argv)
{
	int opt;
	while((opt = getopt(argc, argv, "h")) != -1){
		switch(opt){
		case 'h':
			usage();
			return 0;
		default:
			usage();
			return -1;
		}
	}

	if(optind >= argc){
		cerr << argv[0] << ": IP address is not specified" << endl;
		usage();
		return -1;
	}
	in_addr addr;
	if(!inet_aton(argv[optind], &addr)){
		cerr << argv[0] << ": Invalid IP address" << endl;
		usage();
		return -1;
	}

	CJBLib lib(addr.s_addr);
	unsigned long maxSize;
	JBErrorCode errCode = lib.MaxUpdateProgramSizeRequest(&maxSize);

	if(errCode == JTAG_NO_ERROR)
		cout << maxSize << endl;
	else
		fprintf(stderr, "Failed (code : %08x)\n", (int)errCode);

	return (int)errCode;
}
