#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <glib.h>
#include <glib/gi18n-lib.h>
#include <popt.h>
#include "jblib.h"

void WriteProgress(int status, int param)
{
	printf(_("\rwrite size : %d"), param);
	fsync(1);
}

static int help = 0;
static int usage = 0;
static int verbose = 0;
static char *mode = NULL;
static int te7720 = 0;
static const struct poptOption options[] = {
	{"verbose", '\0', POPT_ARG_NONE, &verbose, 0, _("Verbose output"), NULL},
	{NULL, 'l', POPT_ARG_NONE, &te7720, 0, _("Short hand for --mode=te7720"), NULL},
	{"mode", 'm', POPT_ARG_STRING, &mode, 0, _("Mode selection"), NULL},
	POPT_AUTOHELP
	{NULL, '\0', 0, NULL, 0},
};

int main(int argc, char** argv)
{
	int ret;
	poptContext context;
	in_addr addr;
	struct stat st;
	const char *ip;
	const char *file_name;
	JBErrorCode err;

	context = poptGetContext(NULL, argc, (const char **)argv, options, 0);
	poptSetOtherOptionHelp(context, _("[OPTION...] ip-number file"));
        while ((ret = poptGetNextOpt(context)) != -1) {
                switch(ret) {
                case POPT_ERROR_NOARG:
                case POPT_ERROR_BADOPT:
                case POPT_ERROR_BADNUMBER:
                case POPT_ERROR_OVERFLOW:
                        printf(_("%s: bad argument %s: %s\n"), poptGetInvocationName(context),
                               poptBadOption(context, 0), poptStrerror(ret));
                        poptFreeContext(context);
                        exit(1);
                        break;
                }
        }

        if (help) {
                poptPrintHelp(context, stdout, 0);
                return 0;
        }
        if (usage) {
                poptPrintHelp(context, stdout, 0);
                return 0;
        }

	ip = poptGetArg(context);
	file_name = poptGetArg(context);

	if (!(ip && file_name)) {
		fprintf(stderr, _("%s: bad arguments\n"), poptGetInvocationName(context));
		poptPrintHelp(context, stderr, 0);
		return 1;
	}

	if(!inet_aton(ip, &addr)){
		fprintf(stderr, _("%s: Invalid IP address\n"), poptGetInvocationName(context));
		poptPrintHelp(context, stderr, 0);
		return 1;
	}

	memset(&st, 0, sizeof(st));
	ret = stat(file_name, &st);
	if (ret == -1) {
		fprintf(stderr, _("%s: %s\n"), poptGetInvocationName(context), strerror(errno));
		return 1;
	}
	if (!S_ISREG(st.st_mode)) {
		fprintf(stderr, _("%s: file '%s' is not a regular file\n"),
			poptGetInvocationName(context), file_name);
		return 1;
	}

	if (mode) {
		if (strcmp(mode, "te7720")) {
			fprintf(stderr, _("%s: Unknown mode '%s'\n"),
				poptGetInvocationName(context), mode);
			return 1;
		}
		te7720 = 1;
	}

	CJBLib lib(addr.s_addr);
	err = lib.TargetWriteRequest(file_name,
				     te7720,
				     verbose? WriteProgress : NULL);

	if(verbose) printf("\n");
	if(err != JTAG_NO_ERROR)
		fprintf(stderr, _("%s: Failed: %s (code : %08x)\n"),
			poptGetInvocationName(context), jb_error_get_string(err), err);

        poptFreeContext(context);
	return (int)err;
}
