//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+    FILE : stj_parse.c
//+ SUMMARY : SVFtoJSVF SVF parse function
//+     REV : 1.00 (2005/01/21) 
//+  AUTHOR : M.Nakai (Atmark-Techno,Inc.)
//+    MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//

//+<< INCLUDE >>+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
#include <stdio.h>
#include <string.h>

#include "stj_common.h"
#include "stj_data_cache.h"
#include "stj_parse.h"
#include "stj_process.h"

//#define __SVF_COM_DEBUG
#ifdef __SVF_COM_DEBUG
#define PrintSvfCom(args) LOG_CMSG(CL_GREEN,args "\n")
#else
#define PrintSvfCom(args)
#endif

static char *SVF_COM[]={
  "UNDEFINED",
  "SIR",
  "SDR",
  "HIR",
  "HDR",
  "TIR",
  "TDR",
  "ENDIR",
  "ENDDR",
  "TRST",
  "STATE",
  "RUNTEST",
  "FREQUENCY",
  "PIO",
  "PIOMAP",
  "UNDEFINED",
};

//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : SvfOrder
//+   NOTE : 
//+  PARAM : 
//+ RETURN : "!=0":FirstUsableCharacter,"0":ERROR
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static char SvfOrder(void){
  char ReadByte[2];
  char CommentFlag=0;//"0":Not comment,"1":Now comment
  while(1){
    if(DataCacheReadByte(&ReadByte[0])==-1){
      return 0;
    }
    switch(ReadByte[0]){
    case '!':
      CommentFlag=1;
      break;
    case '/':
      if(CommentFlag==1) break;
      if(DataCachePeekReadByte(&ReadByte[1])==-1){
	return 0;
      }
      if(ReadByte[1]=='/'){
	CommentFlag=1;
      }else{
	return ReadByte[0];
      }
      break;
    case '\r':
      break;
    case '\n':
      CommentFlag=0;
      break;
    case ' ':
      break;
    case '\t':
      break;
    default:
      if(CommentFlag==0) return ReadByte[0];
      break;
    }
  }
  return 0;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : GetCommand
//+   NOTE : 
//+  PARAM : 
//+ RETURN : "0":SUCCESS,"-1":ERROR
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static int GetCommand(char *com){
  int i;
  char Tmp;
  Tmp=SvfOrder();
  if(Tmp==0) return -1;
  com[0]=Tmp;

  for(i=1;i<256;i++){
    if(DataCacheReadByte(&com[i])==-1){
      return -1;
    }
    switch(com[i]){
    case ' ':
      com[i]=0;
      return 0;
    default:
      break;
    }
  }
  return -1;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : SeekCommand
//+   NOTE : 
//+  PARAM : 
//+ RETURN : "xx":CommandID,"-1":ERROR,"0":NO COMMAND
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int SeekCommand(void){
  int i;
  char Command[256];
  memset(Command,0,256);
  if(GetCommand(Command)==-1){
    return 0;
  }

  for(i=SVF_COM_UNDEFINED_START;i!=SVF_COM_UNDEFINED_END;i++){
    if(memcmp(Command,SVF_COM[i],strlen(SVF_COM[i])+1)==0){
      return i;
    }
  }
  LOG_ERR("[SeekCommand] unknown Command : %s\n",Command);
  return -1;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : GetLabel
//+   NOTE : 
//+  PARAM : 
//+ RETURN : "0-1":SUCCESS(1:EndOfLine),"-1":ERROR
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int GetLabel(char *com){
  int i;
  char Tmp;
  Tmp=SvfOrder();
  if(Tmp==0 || Tmp==';') return -1;
  com[0]=Tmp;

  for(i=1;i<256;i++){
    if(DataCacheReadByte(&com[i])==-1){
      return -1;
    }
    switch(com[i]){
    case ';':
      com[i]=0;
      return 1;
    case ' ':
      com[i]=0;
      return 0;
    case '\n':
    case '\r':
      i--;
      break;

    default:
      break;
    }
  }
  return -1;
}

//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int AnalyzeWaitMode(void){
  int ComID;

  DC_Position Pos;
  InfoRUNTEST Info;

  DataCacheSavePosition(&Pos);
  InitInfoRUNTEST(&Info);

  for(;;){
    if((ComID=SeekCommand())<=0){
      DataCacheLoadPosition(&Pos);
      return 0;
    }
    switch(ComID){
    case SVF_COM_RUNTEST:
      ScanRuntestInfo(&Info);
      if(Info.WaitState!=0xff){
	DataCacheLoadPosition(&Pos);
	return 1;
      }
      break;
    default:
      DummyProcess();
      break;
    }
  }
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : SvfParse
//+   NOTE : 
//+  PARAM : 
//+ RETURN : "0":SUCCESS,"-1":ERROR
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int SvfParse(void){
  int ComID;
  int FlagRuleRUNTEST=0;

  DC_Position NextPos;
  InfoRUNTEST InfoRuntest;

  DataCacheSavePosition(&NextPos);
  InitInfoRUNTEST(&InfoRuntest);

  for(;;){
    FlagRuleRUNTEST=RuleProcessRUNTEST(&NextPos,&InfoRuntest,FlagRuleRUNTEST);
    if((ComID=SeekCommand())<=0){
      return ComID;
    }

    switch(ComID){
    case SVF_COM_SIR:
      PrintSvfCom("SIR");
      SvfProcessSIR();
      break;
    case SVF_COM_SDR:
      PrintSvfCom("SDR");
      SvfProcessSDR();
      break;
    case SVF_COM_HIR:
      PrintSvfCom("HIR");
      SvfProcessHIR();
      break;
    case SVF_COM_HDR:
      PrintSvfCom("HDR");
      SvfProcessHDR();
      break;
    case SVF_COM_TIR:
      PrintSvfCom("TIR");
      SvfProcessTIR();
      break;
    case SVF_COM_TDR:
      PrintSvfCom("TDR");
      SvfProcessTDR();
      break;
    case SVF_COM_ENDIR:
      PrintSvfCom("ENDIR");
      SvfProcessENDIR();
      break;
    case SVF_COM_ENDDR:
      PrintSvfCom("ENDDR");
      SvfProcessENDDR();
      break;    
    case SVF_COM_TRST:
      PrintSvfCom("TRST");
      SvfProcessTRST();
      break;
    case SVF_COM_STATE:
      PrintSvfCom("STATE");
      SvfProcessSTATE();
      break;
    case SVF_COM_RUNTEST:
      PrintSvfCom("RUNTEST");
      SvfProcessRUNTEST();
      break;
    case SVF_COM_FREQUENCY:
      PrintSvfCom("FREQUENCY");
      SvfProcessFREQUENCY();
      break;
    case SVF_COM_PIO:
      PrintSvfCom("PIO");
      SvfProcessPIO();
      LOG_ERR("[SvfParse] NOT SUPPORT (PIO) process\n");
      return -1;
    case SVF_COM_PIOMAP:
      PrintSvfCom("PIOMAP");
      SvfProcessPIOMAP();
      LOG_ERR("[SvfParse] NOT SUPPORT (PIOMAP) process\n");
      return -1;
    default:
      LOG_ERR("[SvfParse] NOT SUPPORT (%s) process\n",SVF_COM[ComID]);
      return -1;
    }
  }
  return 0;
}





